#ifndef __List_h__
#define __List_h__

#include "Error.h"
#include <algorithm>
#include <list>

template<class T>
class List
{
    typedef std::list<T> Super;
public:
    typedef typename Super::iterator iterator;
    typedef typename Super::const_iterator const_iterator;

    void append( const T &value )
    {
        _container.push_back( value );
    }

    void appendHead( const T &value )
    {
        _container.push_front( value );
    }

    iterator begin()
    {
        return _container.begin();
    }

    const_iterator begin() const
    {
        return _container.begin();
    }

    iterator end()
    {
        return _container.end();
    }

    const_iterator end() const
    {
        return _container.end();
    }

    iterator insert( iterator position, const T &value )
    {
        return _container.insert( position, value );
    }

    unsigned size() const
    {
        return _container.size();
    }

    bool empty() const
    {
        return _container.empty();
    }

    iterator erase( iterator position )
    {
        return _container.erase( position );
    }

    bool exists( const T &value ) const
    {
        for ( const_iterator it = _container.begin(); it != _container.end(); ++it )
        {
            if ( (*it) == value )
            {
                return true;
            }
        }

        return false;
    }

    void eraseByValue( const T &value )
    {
        for ( iterator it = _container.begin(); it != _container.end(); ++it )
        {
            if ( (*it) == value )
            {
                _container.erase( it );
                return;
            }
        }

        throw Error( Error::VALUE_DOESNT_EXIST_IN_LIST );
    }

    bool operator==( const List<T> &other ) const
    {
        if ( size() != other.size() )
        {
            return false;
        }

        const_iterator it1, it2;

        for ( it1 = begin(), it2 = other.begin(); it1 != end(); ++it1, ++it2 )
        {
            if ( *it1 != *it2 )
            {
                return false;
            }
        }

        return true;
    }

protected:
    Super _container;
};

#endif // __List_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
