#ifndef __SelectableSocket_h__
#define __SelectableSocket_h__

#include "HeapData.h"
#include "ISelectableSocket.h"
#include "ISelector.h"
#include "ISocket.h"

class SelectableSocket : public ISelectableSocket, public ISelector::ISelectable
{
public:
    SelectableSocket( ISocket &socket, ISelector &selector, ISelectableSocket::ISocketDataHandler &socketDataHandler ) :
        _selector( selector ), _socket( socket ), _socketDataHandler( socketDataHandler ), _socketClosed( false )
    {
        _selector.registerToSelector( *this );
    }

    ~SelectableSocket()
    {
        if ( !_socketClosed )
        {
            unregisterFromSelector();
        }
    }

    int descriptor()
    {
        return _socket.descriptor();
    }

    void canRead()
    {
        if ( _socketClosed )
        {
            return;
        }

        HeapData data;

        try
        {
            _socket.receive( data );
            _socketDataHandler.handleData( data );
        }
        catch( const Error &e )
        {
            if ( e.code() == Error::SOCKET_WAS_CLOSED )
            {
                _socketClosed = true;
                unregisterFromSelector();
            }
            else
            {
                throw;
            }
        }
    }

    void closeSocket()
    {
        _socketClosed = true;
        unregisterFromSelector();
        _socket.closeSocket();
    }

    void send( const ConstSimpleData &data )
    {
        _socket.send( data );
    }

    operator ISocket &()
    {
        return _socket;
    }

private:
    ISelector &_selector;
    ISocket &_socket;
    ISelectableSocket::ISocketDataHandler &_socketDataHandler;
    bool _socketClosed;

    void unregisterFromSelector()
    {
        _selector.unregisterFromSelector( *this );
    }
};

#endif // __SelectableSocket_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
