#ifndef __Event_h__
#define __Event_h__

#include "IConstSimpleData.h"
#include "String.h"
#include "HeapData.h"

class Event
{
public:
    Event() : _type( -1 )
    {
    }

    Event( unsigned type ) : _type( type )
    {
    }

    Event( unsigned type, const ConstSimpleData &payload ) : _type( type ), _payload( payload )
    {
    }

    virtual ~Event()
    {
    }

    void serialize( HeapData &serialized )
    {
        serializeType( serialized );
        serializePayloadSize( serialized );
        serializePayload( serialized );
    }

    void deserialize( const ConstSimpleData &serialized )
    {
        unsigned index = 0;

        deserializeType( serialized, index );
        deserializePayload( serialized, index );
    }

    unsigned type() const
    {
        return _type;
    }

    ConstSimpleData payload() const
    {
        return ConstSimpleData( _payload.data(), _payload.size() );
    }

    unsigned payloadSize() const
    {
        return _payload.size();
    }

    unsigned totalSize() const
    {
        return sizeof(_type) + sizeof(unsigned) + _payload.size();
    }

    bool operator==( const Event &other ) const
    {
        return other.type() == type();
    }

    bool operator!=( const Event &other ) const
    {
        return !( other.type() == type() );
    }

    bool operator<( const Event &other ) const
    {
        return type() < other.type();
    }

    Event &operator=( const Event &other )
    {
        _type = other.type();
        _payload = other.payload();

        return *this;
    }

    Event( const Event &other )
    {
        *this = other;
    }

protected:
    unsigned _type;
    HeapData _payload;

private:
    void serializeType( HeapData &serialized )
    {
        serialized = ConstSimpleData( &_type, sizeof(_type) );
    }

    void serializePayloadSize( HeapData &serialized )
    {
        unsigned sizeOfPayload = payloadSize();
        serialized += ConstSimpleData( &sizeOfPayload, sizeof(sizeOfPayload) );
    }

    void serializePayload( HeapData &serialized )
    {
        serialized += _payload;
    }

    void deserializeType( const ConstSimpleData &serialized, unsigned &index )
    {
        memcpy( &_type, serialized.asChar() + index, sizeof(unsigned) );
        index += sizeof(_type);
    }

    void deserializePayload( const ConstSimpleData &serialized, unsigned &index )
    {
        unsigned payloadSize;
        memcpy( &payloadSize, serialized.asChar() + index, sizeof(payloadSize) );
        index += sizeof(payloadSize);

        if ( payloadSize > 0 )
        {
            _payload = ConstSimpleData( serialized.asChar() + index, payloadSize );
        }
        else
        {
            _payload.clear();
        }
        index += _payload.size();
    }
};

#endif // __Event_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
