#ifndef __MockSyncPointResolver_h__
#define __MockSyncPointResolver_h__

#include "Event.h"
#include "ISelector.h"
#include "ISyncPointResolver.h"
#include "Vector.h"

class MockSyncPointResolver : public ISyncPointResolver
{
public:
    class BSyncCallBack
    {
    public:
        virtual void bSyncWasCalled() = 0;
        virtual ~BSyncCallBack() {}
    };

    MockSyncPointResolver()
	{
		wasCreated = false;
		wasDiscarded = false;

        doneAfterNRounds = 10;
        doneCounter = 0;

        lastEventIndex = 0;
        bSyncCallBack = 0;

        nextLastEventIsTimeout = false;
	}

	bool wasCreated;
	bool wasDiscarded;

    ISelector *lastSelector;
    ISocket *lastSocket;

	void mockConstructor( ISelector &selector, ISocket &socket )
	{
		TS_ASSERT( !wasCreated );
		wasCreated = true;

        lastSelector = &selector;
        lastSocket = &socket;
    }

	void mockDestructor()
	{
		TS_ASSERT( wasCreated );
		TS_ASSERT( !wasDiscarded );
		wasDiscarded = true;
	}

    Vector<Event> lastRequested, lastWatched, lastBlocked;
    unsigned lastTimeout;
    String lastLabel;
    AtomicPropositions lastAtomicPropositions;

    BSyncCallBack *bSyncCallBack;

    void go( const Vector<Event> &requested,
             const Vector<Event> &watched,
             const Vector<Event> &blocked,
             unsigned timeoutInSeconds,
             const AtomicPropositions &atomicPropositions,
             const String &label )
    {
        //TS_TRACE( "MockSyncPointResolver go called" );

        lastRequested = requested;
        lastWatched = watched;
        lastBlocked = blocked;
        lastTimeout = timeoutInSeconds;
        lastLabel = label;
        lastAtomicPropositions = atomicPropositions;

        if ( bSyncCallBack )
        {
            bSyncCallBack->bSyncWasCalled();
        }
    }

    EventPredicate *lastBlockPredicate;

    void go( const Vector<Event> &requested,
             const Vector<Event> &watched,
             EventPredicate *blockPredicate,
             unsigned timeoutInSeconds,
             const AtomicPropositions &atomicPropositions,
             const String &label )
    {
        lastRequested = requested;
        lastWatched = watched;
        lastBlockPredicate = blockPredicate;
        lastTimeout = timeoutInSeconds;
        lastLabel = label;
        lastAtomicPropositions = atomicPropositions;
    }

    mutable Vector<Event> nextLastEvent;
    mutable unsigned lastEventIndex;

    Event lastEvent() const
    {
        TSM_ASSERT_LESS_THAN( lastEventIndex, lastEventIndex, nextLastEvent.size() );
        return nextLastEvent[lastEventIndex++];
    }

    unsigned doneAfterNRounds;
    mutable unsigned doneCounter;

    bool done() const
    {
        ++doneCounter;
        return doneCounter >= doneAfterNRounds;
    }

    void handleData( const ConstSimpleData & )
    {
    }

    bool nextLastEventIsTimeout;

    bool lastEventIsTimeout() const
    {
        return nextLastEventIsTimeout;
    }
};

#endif // __MockSyncPointResolver_h__

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
