#include <cxxtest/TestSuite.h>

#include "Engine.h"
#include "MockNonBlockingThreadListenerFactory.h"
#include "MockRegularThreadListenerFactory.h"
#include "MockSelectorFactory.h"
#include "MockTruth.h"
#include "MockWeaverFactory.h"

class MockForEngine :
    public MockSelectorFactory,
	public MockWeaverFactory,
    public MockRegularThreadListenerFactory,
    public MockNonBlockingThreadListenerFactory,
	public MockTruth
{
public:
};

class EngineTestSuite : public CxxTest::TestSuite
{
public:
	MockForEngine *mock;
    MockSelector *selector;
	MockWeaver *weaver;
    MockRegularThreadListener *regularThreadListener;
    MockNonBlockingThreadListener *nonBlockingThreadListener;

	void setUp()
	{
		TS_ASSERT( mock = new MockForEngine );

        selector = &( mock->mockSelector );
		weaver = &( mock->mockWeaver );
        regularThreadListener = &( mock->mockRegularThreadListener );
        nonBlockingThreadListener = &( mock->mockNonBlockingThreadListener );
	}

	void tearDown()
	{
		TS_ASSERT_THROWS_NOTHING( delete mock );
	}

	void test_constructor()
	{
		Engine *engine;

		TS_ASSERT( engine = new Engine );

		TS_ASSERT( selector->wasCreated );

		TS_ASSERT( weaver->wasCreated );
        TS_ASSERT_EQUALS( weaver->lastSelector, selector );

        TS_ASSERT( regularThreadListener->wasCreated );
        TS_ASSERT_EQUALS( regularThreadListener->lastSelector, selector );
        TS_ASSERT_EQUALS( regularThreadListener->lastWeaver, weaver );

        TS_ASSERT( nonBlockingThreadListener->wasCreated );
        TS_ASSERT_EQUALS( nonBlockingThreadListener->lastSelector, selector );
        TS_ASSERT_EQUALS( nonBlockingThreadListener->lastWeaver, weaver );

		TS_ASSERT_THROWS_NOTHING( delete engine );
		TS_ASSERT( selector->wasDiscarded );
        TS_ASSERT( weaver->wasDiscarded );
        TS_ASSERT( regularThreadListener->wasDiscarded );
        TS_ASSERT( nonBlockingThreadListener->wasDiscarded );
	}

	void test_entry_point()
	{
		mock->numberOfTruths = 5;

		TS_ASSERT_THROWS_NOTHING( Engine::main() );

		TS_ASSERT_EQUALS( selector->numberOfSelectCalls, 5U );
	}
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
