#include <cxxtest/TestSuite.h>

#include "MockSelector.h"
#include "MockServerSocket.h"
#include "MockServerSocketFactory.h"
#include "MockSocket.h"
#include "MockThreadListenerFactory.h"
#include "MockWeaver.h"
#include "NonBlockingThreadListener.h"

class MockForNonBlockingThreadListener :
	public MockThreadListenerFactory,
    public MockServerSocketFactory
{
public:
};

class NonBlockingThreadListenerTestSuite : public CxxTest::TestSuite
{
public:
	MockForNonBlockingThreadListener *mock;
	MockServerSocket *serverSocket;
    MockWeaver *weaver;
    MockSelector *selector;
    MockThreadListener *threadListener;
    MockSocket *socket;

	void setUp()
	{
		TS_ASSERT( mock = new MockForNonBlockingThreadListener );
        TS_ASSERT( weaver = new MockWeaver );
        TS_ASSERT( selector = new MockSelector );
        TS_ASSERT( socket = new MockSocket );

        threadListener = mock->registerThreadListener( socket );

        serverSocket = &( mock->mockServerSocket );
	}

	void tearDown()
	{
        threadListener->wasDiscarded = true;

        TS_ASSERT_THROWS_NOTHING( delete socket );
        TS_ASSERT_THROWS_NOTHING( delete selector );
        TS_ASSERT_THROWS_NOTHING( delete weaver );
		TS_ASSERT_THROWS_NOTHING( delete mock );
	}

	void test_constructor()
	{
		NonBlockingThreadListener *nonBlockingThreadListener;

		TS_ASSERT( nonBlockingThreadListener = new NonBlockingThreadListener( *selector, *weaver ) );

		TS_ASSERT( serverSocket->wasCreated );
		TS_ASSERT_EQUALS( serverSocket->lastSelector, selector );
		TS_ASSERT_EQUALS( serverSocket->lastPort, EngineConfiguration::NON_BLOCKING_BTHREAD_MANAGEMENT_PORT );
		TS_ASSERT_EQUALS( serverSocket->lastAccepter, nonBlockingThreadListener );

		TS_ASSERT_THROWS_NOTHING( delete nonBlockingThreadListener );
		TS_ASSERT( serverSocket->wasDiscarded );
	}

    void test_handle_accepted_socket()
    {
        NonBlockingThreadListener nonBlockingThreadListener( *selector, *weaver );

        TS_ASSERT_THROWS_NOTHING( nonBlockingThreadListener.handleAcceptedSocket( *socket ) );

        TS_ASSERT( threadListener->wasCreated );
        TS_ASSERT_EQUALS( threadListener->lastWeaver, weaver );
        TS_ASSERT_EQUALS( threadListener->lastSelector, selector );
        TS_ASSERT_EQUALS( threadListener->lastSocket, socket );
    }
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
