#ifndef __HttpCgiRunner_h__
#define __HttpCgiRunner_h__

#include "BThread.h"
#include "ServerConfiguration.h"
#include "ShellCommandRunner.h"

class HttpCgiRunner : public BThread
{
public:
    HttpCgiRunner() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForCgiRequests();
                storeRequest();
                extractFilenameAndParameters();
                setQueryString();
                runCgi();
                sendOutput();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: HttpCgiRunner" );
            throw e;
        }
    }

private:
    TcpEvent _getRequest;
    String _fileName;
    String _parameterString;
    Map<String, String> _parameters;
    String _fileNameFullPath;
    String _cgiOutput;

    void waitForCgiRequests()
    {
        clearEventVectors();

        _watched.append( IEvent::HTTP_GET_CGI_REQUEST );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void storeRequest()
    {
        _getRequest.constructFromEvent( lastEvent() );
    }

    void extractFilenameAndParameters()
    {
        String requestString( _getRequest.getPacketPayload().asChar(), _getRequest.getPacketPayload().size() );
        _fileName = requestString.tokenize( " " )[1];
        fixSpaceEncoding();

        if ( _fileName.contains( "?" ) )
        {
            _parameterString = _fileName.tokenize( "?" )[1];
            _fileName = _fileName.tokenize( "?" )[0];
        }
        else
        {
            _parameterString = "";
        }

        _fileNameFullPath = String( ServerConfiguration::LOCAL_DIRECTORY ) + _fileName;
    }

    void fixSpaceEncoding()
    {
        static const char WRONG_ENCODING[] = "%20";
        static const char CORRECT_ENCODING[] = " ";

        while ( _fileName.contains( WRONG_ENCODING ) )
        {
            _fileName.replace( WRONG_ENCODING, CORRECT_ENCODING );
        }
    }

    void setQueryString()
    {
        Stringf queryString( "QUERY_STRING=%s", _parameterString.ascii() );

        char *queryStringBuffer = new char[queryString.length() + 1];
        strcpy( queryStringBuffer, queryString.ascii() );

        if ( putenv( queryStringBuffer ) == 0 )
        {
            log( Stringf( "Query string successfully changed: %s", queryStringBuffer ) );
        }
        else
        {
            log( "Failed chaning query string" );
        }
    }

    void runCgi()
    {
        int dontCareReturnCode;
        ShellCommandRunner( _fileNameFullPath.ascii() ).run( dontCareReturnCode, _cgiOutput );
    }

    void log( String message )
    {
        printf( "HttpCgiRunner:\t%s\n", message.ascii() );
    }

    void sendOutput()
    {
        sendHeader();
        sendContent();
    }

    void sendHeader()
    {
        Stringf headerString( "%s%s%s%s%s",
                              "HTTP/1.1 200 Ok\r\n",
                              "Server: BP Server\r\n",
                              "Content-Type: text/html; charset=iso-8859-1\r\n",
                              Stringf( "Content-Length: %u", _cgiOutput.length() ).ascii(),
                              "\r\nConnection: close\r\n\r\n" );

        HeapData header = ConstSimpleData( headerString.ascii(), headerString.length() );

        clearEventVectors();
        _requested.append( TcpEvent( IEvent::DATA_FROM_HTTP_TO_TCP, _getRequest.getIp(), _getRequest.getPort(), header ) );
        _blocked.append( IEvent::HTTP_GET_CGI_REQUEST );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void sendContent()
    {
        clearEventVectors();
        _requested.append( TcpEvent( IEvent::DATA_FROM_HTTP_TO_TCP,
                                     _getRequest.getIp(),
                                     _getRequest.getPort(),
                                     ConstSimpleData( _cgiOutput.ascii(), _cgiOutput.length() ) ) );
        _blocked.append( IEvent::HTTP_GET_CGI_REQUEST );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }
};

#endif // __HttpCgiRunner_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
