#ifndef __HttpDispatcher_h__
#define __HttpDispatcher_h__

#include "BThread.h"
#include "IEvent.h"
#include "TcpEvent.h"

class HttpDispatcher : public BThread
{
public:
    HttpDispatcher() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForRequests();
                classifyRequest();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: HttpDispatcher" );
            throw e;
        }
    }

private:
    void waitForRequests()
    {
        clearEventVectors();

        _watched.append( IEvent::HTTP_DATA_RECEIVED );
        _blocked = allRelevantEventsExcept( IEvent::HTTP_DATA_RECEIVED );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void classifyRequest()
    {
        TcpEvent httpData;
        httpData.constructFromEvent( lastEvent() );

        String request( httpData.getPacketPayload().asChar(), httpData.getPacketPayload().size() );
        log( request );

        if ( request.contains( "GET" ) )
        {
            handleGetRequest();
        }
        else
        {
            generateErrorMessage();
        }
    }

    void handleGetRequest()
    {
        clearEventVectors();

        _requested.append( Event( IEvent::HTTP_GET_REQUEST, lastEvent().payload() ) );
        _blocked = allRelevantEventsExcept( IEvent::HTTP_GET_REQUEST );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void generateErrorMessage()
    {
        clearEventVectors();

        _requested.append( Event( IEvent::HTTP_GENERATE_ERROR, lastEvent().payload() ) );
        _blocked = allRelevantEventsExcept( IEvent::HTTP_GENERATE_ERROR );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    Vector<Event> allRelevantEventsExcept( Event event )
    {
        Vector<Event> result;

        if ( event != IEvent::HTTP_DATA_RECEIVED ) result.append( IEvent::HTTP_DATA_RECEIVED );
        if ( event != IEvent::HTTP_GET_REQUEST ) result.append( IEvent::HTTP_GET_REQUEST );

        return result;
    }

    void log( String message )
    {
        printf( "\tHttp Dispatcher: %s\n", message.ascii() );
    }
};

#endif // __HttpDispatcher_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
