#ifndef __HttpErrorHandler_h__
#define __HttpErrorHandler_h__

#include "BThread.h"
#include "File.h"
#include "IEvent.h"

class HttpErrorHandler : public BThread
{
public:
    HttpErrorHandler() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForErrorRequest();
                storeIpAndPort();
                sendResponse();
                generateFinRequest();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: HttpErrorHandler" );
            throw e;
        }
    }

private:
    enum {
        MAX_FILE_SIZE = 4096,
    };

    String _ip;
    unsigned _port;

    void waitForErrorRequest()
    {
        clearEventVectors();

        _watched.append( IEvent::HTTP_GENERATE_ERROR );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void storeIpAndPort()
    {
        TcpEvent errorRequest;
        errorRequest.constructFromEvent( lastEvent() );
        _ip = errorRequest.getIp();
        _port = errorRequest.getPort();
    }

    void sendResponse()
    {
        String errorString( "HTTP/1.1 404 Not Found\r\n"
                            "Server: BP Server\r\n"
                            "Cache-Control: no-cache,no-store\r\n"
                            "Content-Type: text/html; charset=\%s\r\n"
                            "Connection: close\r\n"
                            "\r\n"
                            "<HTML>\n"
                            "<HEAD><TITLE>404 Not Found</TITLE></HEAD>\n"
                            "<BODY>\n"
                            "<H4>404 Not Found</H4>\n"
                            //                            "File not found. Hopefully, this isn't a BP error...\n"
                            "</BODY>\n"
                            "</HTML>\n" );

        HeapData errorMessage = ConstSimpleData( errorString.ascii(), errorString.length() );

        clearEventVectors();
        _requested.append( TcpEvent( IEvent::DATA_FROM_HTTP_TO_TCP, _ip, _port, errorMessage ) );
        _blocked.append( IEvent::HTTP_GENERATE_ERROR );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void generateFinRequest()
    {
        ConstSimpleData none( 0, 0 );

        clearEventVectors();
        _requested.append( TcpEvent( IEvent::TCP_SEND_FIN, _ip, _port, none ) );
        _blocked.append( IEvent::HTTP_GENERATE_ERROR );

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }
};

#endif // __HttpErrorHandler_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
