#ifndef __Logger_h__
#define __Logger_h__

#include "BThread.h"
#include "File.h"
#include "IEvent.h"
#include "ServerConfiguration.h"

class Logger : public BThread
{
public:
    Logger() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForAnyEvent();
                printEvent();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: Logger" );
            throw e;
        }
    }

private:
    void waitForAnyEvent()
    {
        clearEventVectors();
        _watched = IEvent::allEvents();

        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void printEvent()
    {
        time_t secodnsSinceEpoch = time( NULL );
        struct tm *formattedTime = localtime( &secodnsSinceEpoch );

        Event last = lastEvent();

        printf( "%02u:%02u:%02u - \t%s\n", formattedTime->tm_hour, formattedTime->tm_min, formattedTime->tm_sec, IEvent::eventToString( last ).ascii() );
        fflush( 0 );
    }
};

#endif // __Logger_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
