#ifndef __TcpEvent_h__
#define __TcpEvent_h__

#include "Event.h"

class TcpEvent : public Event
{
public:
    TcpEvent()
    {
    }

    TcpEvent( unsigned type, const String &ip, unsigned short port, const ConstSimpleData &payload ) : Event( type ), _ip( ip ), _port( port ), _packetPayload( payload )
    {
        buildCompletePayload();
    }

    TcpEvent( const TcpEvent &other ) : Event( other.type(), other.payload() ),
                                                            _ip( other.getIp() ),
                                                            _port( other.getPort() ),
                                                            _packetPayload( other.getPacketPayload() )
    {
    }

    void constructFromEvent( const Event &event )
    {
        *((Event *)this) = event;

        unsigned ipLength;
        memcpy( &ipLength, payload().data(), sizeof(ipLength) );
        _ip = String( payload().asChar() + sizeof(ipLength), ipLength );
        memcpy( &_port, payload().asChar() + sizeof(ipLength) + ipLength, sizeof(_port) );

        _packetPayload = ConstSimpleData( payload().asChar() + sizeof(ipLength) + ipLength + sizeof(_port), payload().size() - sizeof(ipLength) - ipLength - sizeof(_port) );
    }

    TcpEvent &operator=( const TcpEvent &other )
    {
        _type = other.type();
        _payload = other.payload();

        _ip = other.getIp();
        _port = other.getPort();
        _packetPayload = other.getPacketPayload();

        return *this;
    }

    String getIp() const
    {
        return _ip;
    }

    unsigned short getPort() const
    {
        return _port;
    }

    ConstSimpleData getPacketPayload() const
    {
        return ConstSimpleData( _packetPayload.data(), _packetPayload.size() );
    }

private:
    String _ip;
    unsigned short _port;
    HeapData _packetPayload;

    void buildCompletePayload()
    {
        unsigned ipLength = _ip.length();
        _payload = ConstSimpleData( &ipLength, sizeof(ipLength) );
        _payload += ConstSimpleData( _ip.ascii(), ipLength );
        _payload += ConstSimpleData( &_port, sizeof(_port) );
        _payload += _packetPayload;
    }
};

#endif // __TcpEvent_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
