#ifndef __TcpPushHandler_h__
#define __TcpPushHandler_h__

#include "BThread.h"
#include "IEvent.h"

class TcpPushHandler : public BThread
{
public:
    TcpPushHandler() : BThread( BThread::REGULAR_THREAD )
    {
    }

    void entryPoint()
    {
        try
        {
            while ( true )
            {
                waitForTcpPushRequest();

                storePushRequest();
                ackPacket();
                waitForAckToBeSent();

                sendDataToApplicationLayer();
            }
        }
        catch ( Error &e )
        {
            e.setUserMessage( "Thread: TcpPushHandler" );
            throw e;
        }
    }

private:
    unsigned _sequenceNumber;
    TcpEvent _pushRequest;

    void storePushRequest()
    {
        _pushRequest.constructFromEvent( lastEvent() );
    }

    void waitForTcpPushRequest()
    {
        clearEventVectors();

        _watched.append( IEvent::TCP_PUSH_REQUEST ) ;
        _blocked = allRelevantEventsExcept( IEvent::TCP_PUSH_REQUEST );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void ackPacket()
    {
        clearEventVectors();

        TcpEvent sendAck( IEvent::TCP_SEND_ACK, _pushRequest.getIp(), _pushRequest.getPort(), _pushRequest.getPacketPayload() );
        _requested.append( sendAck );

        _blocked = allRelevantEventsExcept( IEvent::TCP_SEND_ACK );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    // HeapData extractHttpPayload()
    // {
    //     TcpSegment tcpSegment;
    //     tcpSegment.unserialize( _pushRequest.getPacketPayload() );
    //     return tcpSegment._payload;
    // }

    void sendDataToApplicationLayer()
    {
        clearEventVectors();

        //        HeapData httpPayload = extractHttpPayload();

        //        TcpEvent httpData( IEvent::DATA_FROM_TCP_TO_HTTP, _pushRequest.getIp(), _pushRequest.getPort(), httpPayload );
        TcpEvent httpData( IEvent::DATA_FROM_TCP_TO_HTTP, _pushRequest.getIp(), _pushRequest.getPort(), _pushRequest.getPacketPayload() );
        _requested.append( httpData );

        _blocked = allRelevantEventsExcept( IEvent::DATA_FROM_TCP_TO_HTTP );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    void waitForAckToBeSent()
    {
        clearEventVectors();

        _watched.append( Event( IEvent::TCP_DONE_HADNLING_ACK ) );
        _blocked = allRelevantEventsExcept( IEvent::TCP_DONE_HADNLING_ACK );
        bSync( _requested, _watched, _blocked, IBThread::NO_TIMEOUT );
    }

    Vector<Event> allRelevantEventsExcept( Event event )
    {
        Vector<Event> result;

        if ( event != IEvent::TCP_PUSH_REQUEST ) result.append( IEvent::TCP_PUSH_REQUEST );
        if ( event != IEvent::DATA_FROM_TCP_TO_HTTP ) result.append( IEvent::DATA_FROM_TCP_TO_HTTP );

        return result;
    }
};

#endif // __TcpPushHandler_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
